from openpyxl import load_workbook
import time
import re
import random
import logging
import sys
import json


logger=logging.getLogger("BearSki.DataTable")

def getRowData(data="",datafile=""):
  datalist=data.split(".")
  wb = load_workbook(filename = datafile)
  sheet_ranges = wb[datalist[0]]
  values=sheet_ranges.values
  clumtitle=next(values)
  for rowdata in values:
      if rowdata[0]==datalist[1]:
        if len(datalist)==2:
          logger.debug(u"获取数据源原始数据：标题行 [{0}] 数据行 [{1}]".format(clumtitle,rowdata))
          return clumtitle,rowdata
        else:
          try:
            i=clumtitle.index(datalist[2])
          except ValueError:
            raise DataTableError("数据索引异常 [{0}] 中 {1} 数据异常，请检查".format(data,datalist[2]))
          logger.debug(u"获取数据源原始数据：标题 [{0}] 数据 [{1}]".format(clumtitle[i],rowdata[i]))
          return clumtitle[i],rowdata[i]
  raise DataTableError("没有找到需要获取的数据 [{0}] ，请检查数据是否存在，索引是否正确".format(data))


def generate_json_data(title,RowData):
  re={}
  if type(RowData)==type("str"):
    re[title]=sfun(RowData)
    return re
  else:
    re["dataid"]=""
    re["remark"]=""
    re['detail']={}
    for i in range(0,len(RowData)):
      if title[i]=="DataID":
         re["dataid"]=sfun(RowData[i])
      elif title[i]=="Remark":
        re["remark"]=sfun(RowData[i])
      elif title[i]== None:
        continue
      elif RowData[i] ==None:
        re['detail'][title[i]]=""
      else :
        re['detail'][title[i]]=sfun(RowData[i])
    return re

def generate_data(title,RowData):
  if type(RowData)==type("str"):
    return sfun(RowData)
  else:
    re=[]
    for i in range(0,len(RowData)):
      if title[i]=="DataID":
         continue
      elif title[i]=="Remark":
         continue
      elif title[i]== None:
        continue
      elif RowData[i] ==None:
        re.append("")
      else :
        re.append(sfun(RowData[i]))
    return re

# 后续需要修改代码，提升正则速度，减少正则对比次数
def sfun(re_str):
  nstr=s_fun(re_str)
  if nstr == re_str:
    return s_jl_fun(re_str)
  return nstr

def s_fun(re_str):
  pattern = re.compile(r'\$\{.*?\}')
  longstr=pattern.finditer(str(re_str)) ##需要字符串
  result=str(re_str)
  for value in longstr:
    newdata=runfun(value.group())
    result=result.replace(value.group(),newdata,1)
  return result

def s_jl_fun(re_str):
  # print("in sfun")
  pattern = re.compile(r'\$((json)|(list))\{.*?\}')
  longstr=pattern.finditer(str(re_str)) ##需要字符串
  result=str(re_str)
  for value in longstr:
    result=get_jl(value.group())
    # result=result.replace(value.group(),newdata,1)
  return result

def get_jl(cellstr):
  type_str=cellstr[1:5]
  if type_str == "json":
    json_str=cellstr[6:]
    try:
      rejson=json.loads(json_str)
    except json.decoder.JSONDecodeError: 
      raise DataTableError("数据[{0}],转化json 类型异常".format(json_str))
    return rejson
  if type_str == "list":
    list_str=cellstr[6:-1]
    # print(list_str)
    relist=list_str.split(",")
    re=[]
    for svalue in relist:
      if svalue.find('.int') != -1:
        re.append(int(svalue[:-4]))
        continue
      re.append(svalue)
    return re

def runfun(cellstr):
  if cellstr.find('time.now') != -1:
      newvalue=time.strftime("\"%Y-%m-%d %H:%M:%S\"", time.localtime())
      return newvalue
  elif cellstr.find('random.int')!= -1:
      newvalue=getRand_int(int(cellstr[13:-2]))
      return newvalue
  else:
      return cellstr

def getRand_int(num):
  star=10**(num-1)
  end=10**(num)-1
  # V1.2.18 修改随机数位数计算错误问题
  return str(random.randint(star,end))

def getJsonData(filename="",filepath=""):
  fo = open(filepath+filename, "r+",encoding='utf8')
  jstr=sfun(fo.read())
  redata=json.loads(jstr)
  return redata

class DataTableError(Exception):
  def __init__(self, value):
    self.value = value
  def __str__(self):
    return repr(self.value)


import argparse
from BearSki.Manager import create_project
from BearSki.gui.ApiTestGui import gui_start
from BearSki.utils.arguments import runArg
from BearSki.utils.hartool import HarTool
import BearSki
import json

def main():

  rArg=runArg()
  
  parser = argparse.ArgumentParser(description=BearSki.__description__)
  # parser.add_argument("commond",help="run commond",choices=['createproject','HarParser'])
  parser.add_argument('-v', '--version',help="show version",action="store_true")
  parser.add_argument('commond', choices=['createproject','HarParser','tools'],help="CLI commond")

  group1=parser.add_argument_group('createproject','create templet testproject')
  group1.add_argument('-pn', '--projectname',default='AgaveTestProject',help="testcase file path")

  group2=parser.add_argument_group('HarParser','create testmodel and testcase from har file')
  group2.add_argument('-fp', '--harfilepath',help="parser har file path")
  group2.add_argument('-mp', '--modelpath',default="testdata/model",help="testmodel file path")
  group2.add_argument('-cp','--casepath',default='testcase',help='testcase file path')
  group2.add_argument('-cf','--configfile',default='config.json',help='config file path')

  group3=parser.add_argument_group('tools','test tools')
  group3.add_argument('-g', '--gui',help="api gui test tools",action="store_true")

  args = parser.parse_args()
  f= open(args.configfile)
  conf=json.load(f)
  rArg.setValueFromJson(conf)
  if args.commond:
      if args.commond=='createproject':
        create_project(args.projectname)
      if args.commond=='HarParser':
        if args.configfile:
          if args.harfilepath:
            HarTool(args.harfilepath).createAllCase()
      if args.commond=='tools':
        if args.gui: 
          gui_start()

  # if args.verbasity:
  #   print("打开 verbosity")
if __name__ == "__main__":
  main()




import os
import logging
DRIVER_NAME='d_requests.py'
DRIVER_DOC='''
import requests

def get(url, params=None, **kwargs):
  return requests.get(url,params,**kwargs)

def post(url, data=None, json=None, **kwargs):
  return requests.post(url, data, json, **kwargs)
'''
KEYWORD_NAME='send.py'
KEYWORD_DOC='''
from driver import d_requests
from BearSki.utils.logger import SkiLogger

logger=SkiLogger('keywords.send')

def askbaidu(mod,data):
    logger.info('in ask baidu！')
    r = d_requests.get(url=data)    # 最基本的GET请求
    return r
'''
TESTCASE_NAME='test_send.py'
TESTCASE_DOC='''
# coding=utf-8
import time
import unittest
from BearSki.core import Ski
import logging
class TestSendMessage(unittest.TestCase,Ski):
    def setUp(self):
        self.logger=logging.getLogger('TestSendMessage')
    def tearDown(self):
        pass
    @Ski.case()
    def test_send(self):
        self.logger.info("I'm in test_two test_send")
        # print(self.ski_step_result)
        self.assertTrue(True)
        res=self.step("userkw_sendmsg","get","http://www.baidu.com")
        self.logger.info(res.result)
        self.assertEqual(200,res.result.status_code)
    
    @Ski.case()
    def test_send_use_robotframework_requestlibary(self):
        self.logger.info("I'm in test_two test_send")
        # print(self.ski_step_result)
        # self.assertTrue(True)
        self.step("Create Session","baidu","http://www.baidu.com")
        res=self.step("Get Request","baidu","/")
        self.assertEqual(200,res.result.status_code)

    # @Ski.case()
    # def test_robotframwork_selenium(self):
    #     self.logger.info("I'm in test_two test_send")
    #     self.step("Open Browser","http://www.baidu.com","chrome")
    #     self.step("input text","id=kw","test_robot") 
    #     self.step("click button","id=su")
'''
CONFIG_NAME='config.json'
CONFIG_DOC='''
{
    "m":"allcase",
    "p":"./testcase",
    "n":"",
    "r":"html",
    "o":"./report/result.html",
    "j":"./SkiSetting.json"
}
'''
RUNTEST_NAME='runtest.py'
RUNTEST_DOC='''
import sys

if __name__ == '__main__':
    try:
        from BearSki.CommandLine import CommandLine 
    except ImportError as exc:
        raise ImportError(
            "Couldn't import BearSki. Are you sure it's installed and "
            "available on your PYTHONPATH environment variable? Did you "
            "forget to activate a virtual environment?"
        ) from exc
    CommandLine(sys.argv)
'''
SKISETTING_NAME='SkiSetting.json'
SKISETTING_DOC='''
{
    "routers":{
        "userkw_sendmsg":"keywords.send.askbaidu",
        "Create Session":"RequestsLibrary.RequestsLibrary.create_session",
        "Get Request":"RequestsLibrary.RequestsLibrary.get_request",
        "input text":"Selenium2Library.Selenium2Library.input_text",
        "click button":"Selenium2Library.Selenium2Library.click_button",
        "Open Browser":"Selenium2Library.Selenium2Library.open_browser"
    },
    "global_variable":{
        "BASE_URL":"http://www.baidu.com",
        "logfile_path":"./log/log.log",
        "log_level":"debug"
    }
    
}
'''

def create_testproject(projectname):
    # (logfile_path, logfile_name) = os.path.split(config_json)
    # if projectname and logfile_name:
    isExists=os.path.exists(projectname)
    if not isExists:
      os.makedirs(projectname)
    logger=logging.getLogger('BearSki.createproject')
    logger.info('开始创建测试项目')
    drd=os.path.join(projectname,'driver')
    os.makedirs(drd)
    kwd=os.path.join(projectname,'keywords')
    os.makedirs(kwd)
    tcd=os.path.join(projectname,'testcase')
    os.makedirs(tcd)
    red=os.path.join(projectname,'report')
    os.makedirs(red)

    driverf = os.path.join(drd,DRIVER_NAME)
    df=open(driverf, "w+")
    df.write(DRIVER_DOC)
    df.close

    keywordf=os.path.join(kwd,KEYWORD_NAME)
    kf=open(keywordf, "w+")
    kf.write(KEYWORD_DOC)
    kf.close

    testcasef=os.path.join(tcd,TESTCASE_NAME)
    tf=open(testcasef, "w+")
    tf.write(TESTCASE_DOC)
    tf.close

    configf=os.path.join(projectname,CONFIG_NAME)
    cf=open(configf, "w+")
    cf.write(CONFIG_DOC)
    cf.close

    skisettingf=os.path.join(projectname,SKISETTING_NAME)
    sf=open(skisettingf, "w+")
    sf.write(SKISETTING_DOC)
    sf.close

    runtestf=os.path.join(projectname,RUNTEST_NAME)
    rf=open(runtestf, "w+")
    rf.write(RUNTEST_DOC)
    rf.close

    logger.info('测试项目创建完成')


     
      

  
    




import json
import os
from BearSki.utils.arguments import runArg 
from BearSki.utils.singleton import Singleton
@Singleton
class SkiGlobalData(object):

    def __init__(self):
        self.classes={}
        self.rags=runArg()
        self.set_data=self.__get_conf_data()
        
        
    def get_step_class_instance(self,cls_name):

        if cls_name in self.classes:
            return self.classes[cls_name]
        else:
            return None
    def set_step_class_instance(self,cls_name,cls):
        self.classes[cls_name]=cls
    def get_setting_data(self):
        return self.set_data['routers']
    def __get_conf_data(self):
        f= open(self.rags.jsonfile_path)
        conf=json.load(f)
        return conf
    def get_global_all_data(self):
        return self.set_data['global_variable']
    def get_global_data(self,value):
        try:
            returndata = self.set_data['global_variable'][value]
            return returndata
        except Exception:
            return  None
    def add_global_data(self,data):
        for s in data:
            self.set_data['global_variable'][s]=data[s]
    def get_datatable_config(self):
        returndata = self.set_data['datatable']
        return returndata
    def get_initdata(self,name):
        try:
            returndata=self.set_data['initdata'][name]
            return returndata
        except Exception as e:
            return None



from BearSki.utils.singleton import Singleton
@Singleton
class runArg(object):
  
  def __init__(self,mode='allrun',
                    case_path='./testcase',
                    report_path='./report/result.html',
                    config_path='./config.json',
                    jsonfile_path='./SkiSetting.json',
                    report_mode='text'):
    self.report_mode=report_mode
    self.mode=mode
    self.case_path=case_path
    self.case_name=''
    self.report_path=report_path
    self.config_path=config_path
    self.jsonfile_path=jsonfile_path

  def setValueFromJson(self,jsonstr):
    self.mode=jsonstr['m']
    self.report_mode=jsonstr['r']
    self.report_path=jsonstr['o']
    self.case_name = jsonstr['n']
    self.case_path = jsonstr['p']
    self.jsonfile_path=jsonstr['j']
    if 'report.addtime.now' in jsonstr:
      self.report_add_time=jsonstr['report.addtime.now']
    else:
      self.report_add_time=False
    if 'auto.case.path' in jsonstr:
      self.auto_case_path=jsonstr['auto.case.path']
    else:
      self.auto_case_path=False
    if 'auto.model.path' in jsonstr:
      self.auto_model_path=jsonstr['auto.model.path']
    else:
      self.auto_model_path=False
    
  def getJsonString(self):
    message={
      "report_mode":self.report_mode,
      "mode":self.mode,
      "case_path":self.case_path,
      "case_name":self.case_name,
      "report_path":self.report_path,
      "config_path":self.config_path,
      "json_path":self.jsonfile_path,
      "report.addtime.now":self.report_add_time,
      "auto.case.path":self.auto_case_path,
      "auto.model.path":self.auto_model_path
    }
    return message



# -*- encoding: utf-8 -*-
'''
@File    :   CommandLine.py
@Time    :   2019/11/20 11:28:10
@Author  :   Sirius 
'''
#后续使用BearCLI.py 文件替换命令行实现，后续版本中移除
import sys, getopt
import json
import os
from BearSki.utils.command import command_parser
from BearSki.utils.command import base_command
from BearSki.utils.arguments import runArg
from BearSki.utils.errors import ArgmentError
from BearSki.RunUnittest import RunUnittest
from BearSki.utils.logger import LoggerBaseConfig
import logging 

class CommandLine(object):
    command_list=[
        ["h","help","show help"],
        ["m","mode","run all test or oncase"],
        ["f","configfile","config path"],
        ["p","casepath","run casepath"],
        ["n","casename","run casename"],
        ["r","reportmode","test report ouput mode"],
        ["o","outputpath","test report ouput path"],
        ["j","jsonfile","json file path"]
      ]# 短命令超过1个字符 getopt会出错，暂时遗留问题
    cps=command_parser()
    rarg=runArg()
    def __init__(self,argv):
      # LoggerBaseConfig()._initlogger()
      self.logger=logging.getLogger("BearSki.commondline")
      for cmd in self.command_list:
        help_cmd=base_command(cmd[0],cmd[1],cmd[2])
        self.cps.add_argument(help_cmd)
      self._run(argv)
      
    def _run(self,argv):
      shot_cmd = self.cps.get_shot_arg()
      long_cmd = self.cps.get_long_arg()
      arguments=argv[1:]
      if len(arguments)<2:
        arguments.append(" ") # 命令行参数为空
      try:
          opts, args = getopt.getopt(arguments,shot_cmd,long_cmd)
      except getopt.GetoptError as e:
          # print (e)
          self.logger.error(e)
          sys.exit(2)
      for opt, arg in opts:
          if opt in ('-h',"--help"):
            self._command_h()
            sys.exit()
          elif opt in ("-f", "--configfile"):
            self.rarg.config_path=arg.lstrip() #.lstrip() 去掉多余的空格
            self._command_f(arg.lstrip())
            break
          elif opt in ("-r","--reportmode"):
            self.rarg.report_mode = arg.lstrip()
          elif opt in ("-m","--mode"):
            self.rarg.mode = arg.lstrip()
          elif opt in ("-p","--casepath"):
            self.rarg.case_path=arg.lstrip()
          elif opt in ("-n","--casename"):
            self.rarg.case_name = arg.lstrip()
          elif opt in ("-o","--outputpath"):
            self.rarg.report_path = arg.lstrip()
      self._command_run(self.rarg)
    
    def _command_h(self):
      self.cps.print_help_message()
          
    def _command_f(self,filepath):
       f= open(filepath)
       conf=json.load(f)
       self.rarg.setValueFromJson(conf)
      #  print(self.rarg.getJsonString())

    def _command_run(self,argObject):
      RunUnittest().runTest(argObject)
    
    

    
    
    



      
  


# -*- encoding: utf-8 -*-
'''
@File    :   singleton.py
@Time    :   2019/11/28 11:50:09
@Author  :   Sirius 
'''

class Singleton(object): 
    def __init__(self, cls): 
        self._cls = cls 
    def __call__(self, *args, **kwargs): 
        try: 
            return self._instance 
        except AttributeError: 
            self._instance = self._cls() 
            return self._instance 


import shutil
import sys
from BearSki.template import BasicTestProject
# from BearSki.utils.logger import SkiLogger
# logger=SkiLogger("Manager")
def create_project(projectname):
    BasicTestProject.create_testproject(projectname)
    print("完成测试项目创建：{0}".format(projectname))



from setuptools import setup,find_packages
import src

setup (
    name =src.__title__,
    version = src.__version__,
    packages = find_packages('src'),  # 包含所有src中的包
    package_dir = {'':'src'},   # 告诉distutils包都在src下
    package_data = {
        # 任何包中含有.txt文件，都包含它
        '': ['*.txt'],
        # 包含demo包data文件夹中的 *.dat文件
        'BearSki': ['data/*.dat'],
    },
    description = src.__description__,
    author = src.__author__,
    url = src.__url__,
    license =src.__license__,
    py_modules=['hello'],
    install_requires = ["requests","har2case"],
    entry_points = {
        'console_scripts': [
            'BearSki = BearSki.BearCLI:main'
        ]
    }
)


TESTCASE='''
# coding=utf-8
import time
import unittest
from BearSki.core import Ski
from BearSki.core import DT
import logging
import json

class atest_${modelname}(unittest.TestCase,Ski):

    def setUp(self):
        self.logger=logging.getLogger("atest_${modelname}")
    def tearDown(self):
        pass
    @Ski.case()
    def case_${modelname}(self):
        model_name="${modelname}"
        self.logger.info("in case_${modelname}")
        mjstr=DT.json(model_name+'_model.json')
        res=self.step("requestFromModel",mjstr)
        jstr=DT.json(model_name+"_res.json")
        self.assertEqual(jstr['status'],res.result.status_code)
        self.logger.info("login res {0}".format(res))
'''



# -*- encoding: utf-8 -*-
'''
@File    :   hartool.py
@Time    :   2020/01/29 15:29:32
@Author  :   chenjiusi 
'''
import logging
import json,os
from  har2case.core import HarParser
from BearSki.utils.arguments import runArg 
from BearSki.template import ApiTest_har
logger=logging.getLogger("BearSki.HarTool")
class HarTool(HarParser):
  
    def __init__(self, har_file_path, filter_str=None, exclude_str=None):
        self.har_file_path = har_file_path
        self.filter_str = filter_str
        self.exclude_str = exclude_str or ""
        self._openfiles()
        self.rArg=runArg()
        
    
    # 生成单接口测试模型,_prepare_teststep继承harparser方法
    def _getOneRequest(self,entry_json):
        return self._prepare_teststep(entry_json)
        
    def _openfiles(self):
      fo = open(self.har_file_path, "r+",encoding='utf8')
      jstr=json.loads(fo.read())
      req_list=jstr['log']['entries']
      i=0
      self.harfile={}
      for req_index in req_list:
        request_str=self._getOneRequest(req_index)
        name=request_str['name']
        response_str=self._getResponse(req_index)
        self.harfile[name]={}
        self.harfile[name]['request']=request_str
        self.harfile[name]['response']=response_str
    
    def _getResponse(self,req):
    #   print(req)
      return req['response']

    def createAllCase(self):
        for name in self.harfile:
            result,res=self.getMessage(name)
            self.createTestCase(result,res)
    #功能函数
    def getMessage(self,name):
        print(name)
        req=self.harfile[name]['request']
        res=self.harfile[name]['response']
        return req,res
    
    def createOneCase(self,name):
        req=self.harfile[name]['request']
        res=self.harfile[name]['response']
        self.createTestCase(req,res)
    
    def createTestCase(self,req_str,res):
        # print(req_str)
        newrul=req_str['name']
        # print(req_str)
        modelpath=self.rArg.auto_model_path
        casepath=self.rArg.auto_case_path
        modelname=newrul.replace('/','_')[1:-1]
        isExists=os.path.exists(modelpath)
        if not isExists:
          os.makedirs(modelpath)
        self.writeFile(modelpath+'/'+modelname+'_model.json',json.dumps(req_str, sort_keys=True, indent=4, separators=(',', ': '),ensure_ascii=False))
        self.writeFile(modelpath+'/'+modelname+'_res.json',json.dumps(res, sort_keys=True, indent=4, separators=(',', ': '),ensure_ascii=False))

        testcase=ApiTest_har.TESTCASE
        newcase=testcase.replace("${modelname}",modelname).replace("${model_file_path}",modelpath)
        
        self.writeFile(casepath+'/'+"atest_"+modelname+'.py',newcase)

    def writeFile(self,filename,context):
        fo= open(filename,"w+")
        # print(context)
        fo.write(context)
        fo.close
      
